function [itc, itc2] = com_itc_BS( Y, D, X, rmse )

% Feb. 11, 2018:
% Modified nchoosek
%
% Feb. 12, 2018:
% Added criteria from Babu and Stoica
% IEEE TRANSACTIONS ON SIGNAL PROCESSING, VOL. 60, NO. 9, SEPTEMBER 2012
% pages 4956-4961

% Return:
%    itc : original ITC for fixed dictionary
%    itc2: dictionary is variable, ITC += (m-2)n
%    IT DOES NOT RETURN THE CRITERIA OF Babu and Stoica

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% itc = [AIC HQ BIC RNML1 RNML2]'
% Formulae for the first three criteria are based on 
% H. Ltkepohl, "New Introduction to Multiple Time Series Analysis"
% Springer-Verlag, 2005
% Chapter 5, page 208
% Formulae for the last two criteria are based on
% C.D. Giurcaneanu at al., "Variable selection in linear regression: 
% Several approaches based on normalized maximum likelihood",
% Signal Processing, 91(8):1671-1692, 2011
% Equations (27) and (32)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[m,N] = size(Y);
[~,n] = size(D);

T = m*N;
gof = 2*log(rmse);
fx = (X(:)~=0);
nop = sum( fx );
nop2 = nop + (m-1)*n;
s = round(nop/N); %Can be done better ?!
comb = gammaln(n+1) - gammaln(s+1) - gammaln(n-s+1);

itc = zeros(6,1);
itc2 = zeros(6,1);

%%%%%%%%%%% BABU AND STOICA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nn = nop2-1;
NN = T;
sigma2 = rmse^2;
%Eq. (33) in the paper: 
temp1 = nn*log(NN);
temp2 = -(nn+1)*log(sigma2);
BICBS1 = NN*log(sigma2) + max(temp1, temp2);
BICSB1 = BICBS1/NN; 
%Eq. (34) in the paper:
% BICSB2 = (NN-nn-1)*log(sigma2) + nn*log(NN);
itc2(6) = BICSB1;

nn = nop-1;
temp1 = nn*log(NN);
temp2 = -(nn+1)*log(sigma2);
BICBS1 = NN*log(sigma2) + max(temp1, temp2);
BICSB1 = BICBS1/NN; 
itc(6) = BICSB1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%AIC
itc(1) = gof + (2/T)*nop;
itc2(1) = gof + (2/T)*nop2;

%HQ
itc(2) = gof + (2*log(log(T))/T)*nop;
itc2(2) = gof + (2*log(log(T))/T)*nop2;

%BIC
itc(3) = gof + (log(T)/T)*nop;
% itc2(3) = gof + (log(T)/T)*nop2 + (2*N/T)*log(nchoosek(n,s));
itc2(3) = gof + (log(T)/T)*nop2 + (2*N/T)*comb;

%RNML
%de = 1;
de = 0;
Xbet = [];
X2bet = [];
for ind=1:size(X,2),
    xx = X(:,ind);
    fxx = find(xx~=0);
    xx = xx(fxx);
    DD = D(:,fxx);
    Xbet = [Xbet; DD*xx];
    DD2 = DD'*DD;
    X2bet = [X2bet; DD2*xx];
    %de = de*det(DD2);       
    de = de - log(det(DD2));
end
Xbetnorm = Xbet'*Xbet;
X2betnorm = X2bet'*X2bet;

%RNML1
term = zeros(4,1); 
term(1) = (T-nop)*log(rmse^2/(T-nop));
term(2) = nop*log( Xbetnorm/T/nop );
term(3) = log( nop*(T-nop) );
itc(4) = sum(term);

term2 = zeros(4,1); 
term2(1) = (T-nop2)*log(rmse^2/(T-nop2));
term2(2) = nop2*log( Xbetnorm/T/nop2 );
term2(3) = log( nop2*(T-nop2) );
% itc2(4) = sum(term2) + 2*N*log(nchoosek(n,s));
itc2(4) = sum(term2) + 2*N*comb;

%RNML2
term(2) = nop*log( X2betnorm/T/nop );
%term(4) = -log(de);
term(4) = de;
itc(5) = sum(term);

term2(2) = nop2*log( X2betnorm/T/nop2 );
%term2(4) = -log(de);
term2(4) = de;
% itc2(5) = sum(term2) + 2*N*log(nchoosek(n,s));
itc2(5) = sum(term2) + 2*N*comb;

% %RNML3
% term(2) = nop*log( norm(X(:),1)^2/T/nop );
% term(4) = nop*log( 2*exp(1)/pi/nop );
% term(5) = 

end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

