% Test script for ITC-ADL
% (Don't forget to download the folder DL from https://github.com/dl-book/dl-book
% and to include it in the path together with all its subfolders.)

% BD 16.02.2019

%-------------------------------------------------------------
% Input data:
m = 64;         % signal length
ntrue = 128;    % true dictionary size
nmin = 64;      % minimum considered dictionary size
nint = [80 180]; % interval for the initial size of the dictionary
strue = 6;      % sparsity level
snr = 20;       % SNR values
itc = 13;       % ITC codes (11 for EBIC, 13 for ERNML)

N = 4000;       % number of training signals
Ntest = 1000;   % number of test signals
iters = 200;    % DL iterations
iter_check = 5; % check ITC only at iterations multiples of this number
nminus = 5;     % how much n decreases (when it decreases)
nplus = 5;      % how much n grows (when it grows)
ncand = 20;     % number of size candidates for ITC
%-------------------------------------------------------------

% Info for AK-SVD
update = 'aksvd';       % set the dictionary update method
replatom = 'random';    % unused atoms are replaced with random vectors
params = {};

% Signal generation (learn+test)
[~, Dtrue, Y] = gen_synth_data(m, ntrue, N+Ntest, strue, snr);
Ytest = Y(:,N+1:end);     % test signals
Y = Y(:,1:N);             % learning signals
ninit = randi(nint, 1);   % random initialization of the size
D0 = normc(randn(m,ninit)); % initial dictionary

% RMSE with true dictionary
Xt = omp(Y, Dtrue, strue);
rmset = norm(Y-Dtrue*Xt, 'fro')/sqrt(m*N);
fprintf('True dictionary error on training data: %.4g\n', rmset)
Xt = omp(Ytest, Dtrue, strue);
rmset = norm(Ytest-Dtrue*Xt, 'fro')/sqrt(m*Ntest);
fprintf('True dictionary test data: %.4g\n', rmset)

% ITC-ADL - adaptive DL algorithm
tic
[D, ~, nopt, vn, rmsev] = itc_adl(Y, D0, strue, nmin, iters, iter_check, nminus, nplus, itc, ncand);
toc
fprintf('ITC-ADL dictionary size: %d\n', nopt(end))
fprintf('ITC-ADL training error: %.4g\n', rmsev(end))

% ITC-ADL results on test signals
X = omp(Ytest, D, strue);     % designed dictionary (adaptive size)
rmse = norm(Ytest-D*X, 'fro')/sqrt(m*Ntest);
fprintf('ITC-ADL test error: %.4g\n', rmse)

% fixed size DL with AK-SVD
D0n = normc(randn(m,ntrue));  % random initial dictionary
tic
[Dn, ~, errsn] = DL(Y, D0n, strue, iters, str2func(update), params, 'replatom', replatom);
toc
Xn = omp(Y, Dn, strue);
rmsen = norm(Y-Dn*Xn, 'fro')/sqrt(m*N);
fprintf('AK-SVD training error: %.4g\n', rmsen)

% AK-SVD results on test signals
Xn = omp(Ytest, Dn, strue);   % designed dictionary (fixed size)
rmsen = norm(Ytest-Dn*Xn, 'fro')/sqrt(m*Ntest);
fprintf('AK-SVD test error: %.4g\n', rmsen)
